@@ -583,45 +583,33 @@ class Image(pyvips.VipsObject):
         """
         format_string = _to_bytes(format_string)
 
+        filename = vips_lib.vips_filename_get_filename(format_string)
+
         pointer = vips_lib.vips_filename_get_options(format_string)
         options = _to_string_copy(pointer)
 
-        pointer = vips_lib.vips_foreign_find_save_buffer(format_string)
-        if pointer == ffi.NULL:
-            raise Error('unable to write to buffer')
-        name = _to_string(pointer)
+        pointer = ffi.NULL
+        if at_least_libvips(8, 9):
+            vips_lib.vips_error_freeze()
+            pointer = vips_lib.vips_foreign_find_save_target(filename)
+            vips_lib.vips_error_thaw()
 
-        return pyvips.Operation.call(name, self,
-                                     string_options=options, **kwargs)
+        if pointer != ffi.NULL:
+            name = _to_string(pointer)
+            target = pyvips.Target.new_to_memory()
+            pyvips.Operation.call(name, self, target,
+                                  string_options=options, **kwargs)
+            buffer = target.get("blob")
+        else:
+            pointer = vips_lib.vips_foreign_find_save_buffer(filename)
+            if pointer == ffi.NULL:
+                raise Error('unable to write to buffer')
 
-    def write_to_memory(self):
-        """Write the image to a large memory array.
+            name = _to_string(pointer)
+            buffer = pyvips.Operation.call(name, self,
+                                           string_options=options, **kwargs)
 
-        A large area of memory is allocated, the image is rendered to that
-        memory array, and the array is returned as a buffer.
-
-        For example, if you have a 2x2 uchar image containing the bytes 1, 2,
-        3, 4, read left-to-right, top-to-bottom, then::
-
-            buf = image.write_to_memory()
-
-        will return a four byte buffer containing the values 1, 2, 3, 4.
-
-        Returns:
-            buffer
-
-        Raises:
-            :class:`.Error`
-
-        """
-
-        psize = ffi.new('size_t *')
-        pointer = vips_lib.vips_image_write_to_memory(self.pointer, psize)
-        if pointer == ffi.NULL:
-            raise Error('unable to write to memory')
-        pointer = ffi.gc(pointer, glib_lib.g_free)
-
-        return ffi.buffer(pointer, psize[0])
+        return buffer
 
     def write_to_target(self, target, format_string, **kwargs):
         """Write an image to a target.
@@ -672,6 +660,35 @@ class Image(pyvips.VipsObject):
         return pyvips.Operation.call(name, self, target,
                                      string_options=options, **kwargs)
 
+    def write_to_memory(self):
+        """Write the image to a large memory array.
+
+        A large area of memory is allocated, the image is rendered to that
+        memory array, and the array is returned as a buffer.
+
+        For example, if you have a 2x2 uchar image containing the bytes 1, 2,
+        3, 4, read left-to-right, top-to-bottom, then::
+
+            buf = image.write_to_memory()
+
+        will return a four byte buffer containing the values 1, 2, 3, 4.
+
+        Returns:
+            buffer
+
+        Raises:
+            :class:`.Error`
+
+        """
+
+        psize = ffi.new('size_t *')
+        pointer = vips_lib.vips_image_write_to_memory(self.pointer, psize)
+        if pointer == ffi.NULL:
+            raise Error('unable to write to memory')
+        pointer = ffi.gc(pointer, glib_lib.g_free)
+
+        return ffi.buffer(pointer, psize[0])
+
     def write(self, other):
         """Write an image to another image.
 
@@ -1180,6 +1197,65 @@ class Image(pyvips.VipsObject):
         else:
             return pyvips.Operation.call('bandjoin', [self] + other)
 
+    def atan2(self, other):
+        return _call_enum(self, other, 'math2', 'atan2')
+
+    def get_n_pages(self):
+        """Get the number of pages in an image file, or 1.
+
+        This is the number of pages in the file the image was loaded from, not
+        the number of pages in the image.
+
+        To get the number of pages in an image, divide the image height by
+        the page height.
+        """
+        if at_least_libvips(8, 8):
+            return vips_lib.vips_image_get_n_pages(self.pointer)
+        else:
+            return self.get("n-pages") \
+                if self.get_typeof("n-pages") != 0 else 1
+
+    def get_page_height(self):
+        """Get the page height in a many-page image, or height.
+        """
+        if at_least_libvips(8, 8):
+            return vips_lib.vips_image_get_page_height(self.pointer)
+        else:
+            if self.get_typeof("page-height") != 0:
+                page_height = self.get("page-height")
+            else:
+                page_height = self.height
+
+            if page_height > 0 and \
+                page_height <= self.height and \
+                    self.height % page_height == 0:
+                return page_height
+            else:
+                return self.height
+
+    def pagesplit(self):
+        """Split an N-page image into a list of N separate images.
+        """
+        page_height = self.get_page_height()
+        return [self.crop(0, y, self.width, page_height)
+                for y in range(0, self.height, page_height)]
+
+    def pagejoin(self, other):
+        """Join a set of pages vertically to make a multipage image.
+
+        Also sets the page-height property on the result.
+        """
+        if not isinstance(other, list):
+            other = [other]
+
+        n_pages = 1 + len(other)
+
+        image = Image.arrayjoin([self] + other, across=1)
+        image = image.copy()
+        image.set_type(GValue.gint_type, 'page-height', image.height / n_pages)
+
+        return image
+
     def composite(self, other, mode, **kwargs):
         """Composite a set of images with a set of modes."""
         if not isinstance(other, list):
@@ -1259,6 +1335,30 @@ class Image(pyvips.VipsObject):
         """Return the inverse tangent of an image in degrees."""
         return self.math('atan')
 
+    def sinh(self):
+        """Return the hyperbolic sine of an image in radians."""
+        return self.math('sinh')
+
+    def cosh(self):
+        """Return the hyperbolic cosine of an image in radians."""
+        return self.math('cosh')
+
+    def tanh(self):
+        """Return the hyperbolic tangent of an image in radians."""
+        return self.math('tanh')
+
+    def asinh(self):
+        """Return the inverse hyperbolic sine of an image in radians."""
+        return self.math('asinh')
+
+    def acosh(self):
+        """Return the inverse hyperbolic cosine of an image in radians."""
+        return self.math('acosh')
+
+    def atanh(self):
+        """Return the inverse hyperbolic tangent of an image in radians."""
+        return self.math('atanh')
+
     def log(self):
         """Return the natural log of an image."""
         return self.math('log')
